p, option, select, input, textarea {
  font-family: Helvetica, Verdana, Arial, sans-serif;
  font-size: 8pt;
  font-style: normal;
  font-variant: normal;
  font-weight: normal;
}

input[type="submit"], select {
  height: 14pt;
}

textarea {
  font-family: monospace;
  border: 0;
  margin: 0;
}

a.opener {
  text-decoration: none;
}

td.control img {
  border: none;
  margin: 2px;
  padding: 0;
}

td.control {
  vertical-align: middle;
  line-height: 1px;
  font-size: 1px;
}

a.opener img, h3 img {
  border: none;
  margin-top: 5px;
  margin-bottom: -5px;
  margin-left: 5px;
}

h2 {
  font-family: Helvetica, Verdana, Arial, sans-serif;
  font-size: 12pt;
  font-style: normal;
  font-variant: small-caps;
  font-weight: bold;
}

h1 {
  font-family: Helvetica, Verdana, Arial, sans-serif;
  font-size: 14pt;
  font-style: normal;
  font-weight: bold;
}

h2 img {
  border: none;
  margin-right: 10px;
}

h3 {
  font-family: Helvetica, Verdana, Arial, sans-serif;
  font-size: 10pt;
  font-style: normal;
  font-variant: normal;
  font-weight: bold;
  background-color: #cccccc;
  margin-bottom: 0;
  margin-top: 10px;
  padding-top: 2px;
  padding-bottom: 3px;
}

div.fetch {
  border-top: solid #999999 1px;
  width: 100%;
  background-color: #ddddcc;
  margin: 0;
}

div.fetch th {
  font-family: Helvetica, Verdana, Arial, sans-serif;
  font-size: 9pt;
  font-style: normal;
  font-variant: normal;
  font-weight: bold;
}

div.fetch td {
  font-family: Helvetica, Verdana, Arial, sans-serif;
  font-size: 9pt;
  font-style: normal;
  font-variant: normal;
  font-weight: normal;
}

.header {
  margin-left: 10pt;
}

.saver a {
  font-size: 9pt;
  font-weight: normal;
  text-decoration: none;
  color: blue;
}

.saver a:hover {
  color: red;
}

form {
  margin: 0;
  padding: 0;
}

td.tdcol1 {
  width: 100px;
}

td.tdcol2 input {
  width: 550px;
}

textarea.fetch_ta {
  width: 680px;
}

textarea.comment_ta {
  width: 190px;
  font-family: Helvetica, Verdana, Arial, sans-serif;
  font-style: italic;
  font-size: 6.5pt;
}

td.tdc1 {
  margin: 0;
  padding: 3px;
  background-color: #aaddaa;
}

td.tdc2 {
  margin: 0;
  padding: 3px;
  background-color: #ddaaaa;
}

td.tdc3 {
  margin: 0;
  padding: 3px;
  background-color: #aaaadd;
}

tr.tdc {
  margin: 0;
  padding: 0;
}

table.tdc {
  margin: 0;
  padding: 0;
  border-collapse: collapse;
}

table.inner_tdc {
  margin: 0;
  padding: 0;
  border-collapse: separate;
  border-spacing: 2px;
}

table.inner_tdc tr, table.inner_tdc td, table.inner_tdc textarea {
  margin: 0;
  padding: 0;
  height: 100%;
}

div.resizer {
  height: 3px;
  margin: 0;
  margin-top: -2px;
  margin-left: 2px;
  margin-right: 2px;
  width: 99.5%;
  background-color: #ddddcc;
  border-style: outset;
  border-top-width: 2px;
  border-bottom-width: 2px;
  border-left-width: 0;
  border-right-width: 0;
  font-size: 1px;
  cursor: n-resize;
}

img.pfeil {
  padding-bottom: 4px;
}